import unittest
import os
import sys
# make sure we use the devel version first
sys.path.insert(0,os.path.dirname(os.path.realpath(__file__))+'/..')
from hrds.hrds import HRDS
import os

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# Copyright Jon Hill, University of York, jon.hill@york.ac.uk

# this is a bit kludgy, but should work on all platforms and test frameworks

test_dir = os.path.split(__file__)[0]

base_raster = os.path.join(test_dir, "base.tif")
layer1 = os.path.join(test_dir, "layer1.tif")
layer2 = os.path.join(test_dir, "layer2.tif")


class TestHRDS(unittest.TestCase):
    """Tests the hrds.hrds.HRDS class"""

    def test_single_raster(self):
        """ A simple single raster solution"""
        
        bathy = HRDS(base_raster)
        bathy.set_bands()

        expected = [1.0,1.0,1.0,1.0,1.0] # uniform raster
        points = ([5, 5],
                  [40, 50],
                  [28, 32],
                  [13.5, 20],
                  [32.5, 45],
                  )
        for p, e in zip(points, expected):
            self.assertAlmostEqual(bathy.get_val(p), e, delta=0.1)



    def test_simple_rasters(self):
        """ Three layer test. Base layer is 100x100 with dx of (2.5,2).
            Origin is at (0,0)
            Layer 1 is 50x75, dx of (0.5,0.5), origin at (10,10)
            Layer 2 is 25x25, dx of (0.1,0.1), origin at (30,35)
            We set a distance buffer of 7 in layer 1 and 5 in layer 2
            BaseLayer has a value of 1, layer 1 of 2 and layer 2 of three.
            We ask for points around all layers and in the buffer and check against
            those expected.
        """

        bathy = HRDS(base_raster, rasters=(layer1, layer2), distances=(7, 5), saveBuffers=True)
        bathy.set_bands()
        points = ([5, 5],  # 1
                  [40, 50],  # 3
                  [28, 32],  # 2
                  [13.5, 20],  # 1.5
                  [32.5, 45],  # 2.5
                  )
        expected = [1.0, 3.0, 2.0, 1.5, 2.5]
        for p, e in zip(points, expected):
            self.assertAlmostEqual(bathy.get_val(p), e, delta=0.1)
        # check that the buffer files exists
        self.assertTrue(os.path.exists(os.path.join(test_dir, "layer1_buffer.tif")))
        self.assertTrue(os.path.exists(os.path.join(test_dir, "layer2_buffer.tif")))
        # delete them
        os.remove(os.path.join(test_dir, "layer1_buffer.tif"))
        os.remove(os.path.join(test_dir, "layer2_buffer.tif"))


class RealDataTest(unittest.TestCase):

    def test_real_world(self):
        """ Mix of GEBCO, EMODnet and UK Gov just off the
        Norfolk coast.
        Projected to UTM 30, so everything in m.
        """
        bathy = HRDS(os.path.join(test_dir,"real_data/gebco_uk.tif"),
                     rasters=(os.path.join(test_dir,"real_data/emod_utm.tif"),
                              os.path.join(test_dir,"real_data/inspire_data.tif")),
                     distances=(700, 200))
        bathy.set_bands()
        """
        Our data:
        X	Y	emod_utm  gebco_uk  inspire_da
        842996.	5848009.     	  -21.318
        834009.	5848207.	  -25.289
        832856.	5848273. -32.76	  -28.598
        828840.	5848306. -5.884	  -13.466
        823178.	5848503. -18.13	  -13.215
        822941.	5848511. -17.56	  -11.241  -14.554
        822762.	5848285. -5.517	  -11.241
        822634.	5848528. -6.757	  -11.241  -8.316
        822447.	5848684. -9.098	  -11.241  -10.60

"""
        points = ([842996., 5848009.],
                  [834009., 5848207.],
                  [832856., 5848273.],
                  [828840., 5848306.],
                  [823178., 5848503.],
                  [822941., 5848511.],
                  [822762., 5848285.],
                  [822634., 5848528.],
                  [822447., 5848684.],
                  )
        expected = [-21.318, #
                    -25.289,
                    -28.6,  # in the buffer, so mostly gebco
                    -5.884,
                    -18.13,
                    -17.0,  # in hi-res area, but mostly emod
                    -5.517,
                    -8.316,  # hi res are only
                    -10.60,  # as above
                    ]
        for p, e in zip(points, expected):
            self.assertAlmostEqual(bathy.get_val(p), e, delta=0.75)


class RealDataTest_limited(unittest.TestCase):

    def test_real_world_limit(self):
        """ Mix of GEBCO, EMODnet and UK Gov data just off the
        Norfolk coast.
        Projected to UTM 30, so everything in m.
        """
        bathy = HRDS(os.path.join(test_dir,"real_data/gebco_uk.tif"),
                     rasters=(os.path.join(test_dir,"real_data/emod_utm.tif"),
                              os.path.join(test_dir,"real_data/inspire_data.tif")),
                     distances=(700, 200),
                     minmax=[[None,-25],[None,-10],[None,None]])
        bathy.set_bands()
        """
        Our data:
        X	Y	emod_utm  gebco_uk  inspire_da
        842996.	5848009.     	  -21.318
        834009.	5848207.	  -25.289
        832856.	5848273. -32.76	  -28.598
        828840.	5848306. -5.884	  -13.466
        823178.	5848503. -18.13	  -13.215
        822941.	5848511. -17.56	  -11.241  -14.554
        822762.	5848285. -5.517	  -11.241
        822634.	5848528. -6.757	  -11.241  -8.316
        822447.	5848684. -9.098	  -11.241  -10.60

"""
        points = ([842996., 5848009.],
                  [834009., 5848207.],
                  [832856., 5848273.],
                  [828840., 5848306.],
                  [823178., 5848503.],
                  [822941., 5848511.],
                  [822762., 5848285.],
                  [822634., 5848528.],
                  [822447., 5848684.],
                  )
        expected = [-25.0,  # -21.318 - limited!
                    -25.289,
                    -28.6,  # in the buffer, so mostly gebco
                    -10,  # -5.884 so limited!
                    -18.13,
                    -17.0,  # in hi-res area, but mostly emod
                    -10,  # 5.517, limited
                    -8.316,  # hi res are only
                    -10.60,  # as above
                    ]
        for p, e in zip(points, expected):
            self.assertAlmostEqual(bathy.get_val(p), e, delta=0.75)


if __name__ == '__main__':
    unittest.main()



import unittest
import os
import sys
# make sure we use the devel version first
sys.path.insert(0,os.path.dirname(os.path.realpath(__file__))+'/..')
from hrds.raster import RasterInterpolator, CoordinateError, RasterInterpolatorError
from numpy import array, ones

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# Copyright Jon Hill, University of York, jon.hill@york.ac.uk


test_file_name1 = os.path.join(os.path.split(__file__)[0], "test_raster.asc")


class TestRasterInterpolator(unittest.TestCase):
    """Tests the hrds.raster.RasterInterpolator class"""
    def setUp(self):
        return

    def tearDown(self):
        return

    def test_simple_interpolation(self):
        """ Very simple test with a raster object like thus:
             1  2  3  4
             5  6  7  8
             9  10 11 12
             13 14 15 16

            LLC is 0,0 and upper right is 4,4.
            The data are stored in cell centres and we ask for a few coords
            """
        rci = RasterInterpolator(test_file_name1)
        point1 = [0.0, 0.0] # should error
        point2 = [1.5, 2.0] # should return 8
        point3 = [2.0, 3.0] # should return 4.5
        point4 = [3,1] # should return 13.5
        point5 = [1.999999, 2.999999] # should return nearly 4.5
        point6 = [0.5, 0.5] # should return 13 (llc)
        point7 = [1.0, 3.49] # should be 1.5
        self.assertRaises(RasterInterpolatorError, rci.get_val, point2)
        rci.set_band()
        self.assertEqual(rci.get_val(point2),8.0)
        self.assertAlmostEqual(rci.get_val(point3),4.5)
        self.assertAlmostEqual(rci.get_val(point5),4.5,5)
        self.assertEqual(rci.get_val(point4),13.5)
        self.assertEqual(rci.get_val(point6),13)
        self.assertAlmostEqual(rci.get_val(point7),1.5, delta=0.1)
        self.assertRaises(CoordinateError,rci.get_val, point1)

    def test_point_in(self):
        """ Very simple test with a raster object like thus:
             1  2  3  4
             5  6  7  8
             9  10 11 12
             13 14 15 16

            LLC is 0,0 and upper right is 4,4.
            we ask for point in, outside and on boundary
            """
        rci = RasterInterpolator(test_file_name1)
        point1 = [0.0, 0.0] # should return False
        point2 = [2.0, 2.0] # should return True
        point3 = [-1.0, -1.0] # should return False
        point4 = [0.501, 0.501] # should return True
        point5 = [0.0001, 0.0001] # False - outside first cell centre
        rci.set_band()
        self.assertFalse(rci.point_in(point1))
        self.assertTrue(rci.point_in(point2))
        self.assertFalse(rci.point_in(point3))
        self.assertTrue(rci.point_in(point4))
        self.assertFalse(rci.point_in(point5))

    @unittest.skipUnless(os.path.isfile("tests/real_data/gebco_uk.tif"),
                         "Skipping as proprietary data missing.")
    def test_real_data(self):
        """Using gebco cut out to test interpolation
        """
        raster_file = "tests/real_data/gebco_uk.tif"
        rci = RasterInterpolator(raster_file)
        rci.set_band()
        points = ([842996., 5848009.],
                  [834009., 5848207.],
                  [832856., 5848273.],
                  [828840., 5848306.],
                  [823178., 5848503.],
                  )
        expected = [-21.3,
                    -25.3,
                    -28.6,
                    -13.5,
                    -13.2,
                    ]
        for p,e in zip(points, expected):
            self.assertAlmostEqual(rci.get_val(p),e,delta=0.75)

    def test_simple_interpolation_limits(self):
        """ Very simple test with a raster object like thus:
             1  2  3  4
             5  6  7  8
             9  10 11 12
             13 14 15 16

            LLC is 0,0 and upper right is 4,4.
            But here we set upper and lower limits
            """
        rci = RasterInterpolator(test_file_name1,minmax=[2,10])
        point1 = [0.0, 0.0] # should error
        point2 = [1.5, 2.0] # should return 8
        point3 = [2.0, 3.0] # should return 4.5
        point4 = [3,1] # should return 13.5, limited to 10
        point5 = [1.999999, 2.999999] # should return nearly 4.5
        point6 = [0.5, 0.5] # should return 13 (llc), limited to 10
        point7 = [1.0, 3.49] # should be 1.5, but limited to 2
        self.assertRaises(RasterInterpolatorError, rci.get_val, point2)
        rci.set_band()
        self.assertEqual(rci.get_val(point2),8.0)
        self.assertAlmostEqual(rci.get_val(point3),4.5)
        self.assertAlmostEqual(rci.get_val(point5),4.5,5)
        self.assertEqual(rci.get_val(point4),10)
        self.assertEqual(rci.get_val(point6),10)
        self.assertEqual(rci.get_val(point7),2)
        self.assertRaises(CoordinateError,rci.get_val, point1)


if __name__ == '__main__':
    unittest.main()



import unittest
import os
import sys
# make sure we use the devel version first
sys.path.insert(0,os.path.dirname(os.path.realpath(__file__))+'/..')
from hrds.raster_buffer import CreateBuffer
from hrds.raster import RasterInterpolator, CoordinateError, RasterInterpolatorError

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# Copyright Jon Hill, University of York, jon.hill@york.ac.uk

test_dir = os.path.split(__file__)[0]

test_file_name1 = os.path.join(test_dir, "test_raster_large.tif")
test_file_name2 = os.path.join(test_dir, "test_raster_nodata_centre.tif")

temp_file = "temp.tif"


class TestBufferCreator(unittest.TestCase):
    """Tests the hrds.buffer.CreateBuffer class"""
    def setUp(self):
        return

    def tearDown(self):
        # remove temp file
        #os.remove(temp_file)
        return

    def test_simple_distance(self):
        """ Very simple test with a raster object like thus:
             1  2  3  4
             5  6  7  8
             9  10 11 12
             13 14 15 16
            (on a 10x10 grid, not 4x4 as above...only so much space
            in comments!)

            LLC is 0,0 and upper right is 4,4. (hence dx is 0.25)
            The data are stored in cell centres and we ask for a few coords.
            We check that the value is 1 at the distance, 0 at the boundary.
            Finally, test that the file is written and readable
            """
        rbuff = CreateBuffer(test_file_name1, 1.5)
        point1 = [0.2, 3.85]  # should return <0.25 (resolution)
        point2 = [1.7, 2]  # should be 1
        point3 = [2, 2]  # should be 1
        point4 = [0.85, 2]  # should be 0.5
        rbuff.make_buffer(temp_file)
        # we now read in the buffer using the rasterinterpolator class
        rci = RasterInterpolator(temp_file)
        rci.set_band()
        self.assertAlmostEqual(rci.get_val(point1), 0.0, delta=0.03)
        self.assertEqual(rci.get_val(point2), 1.0)
        self.assertEqual(rci.get_val(point3), 1.0)
        self.assertAlmostEqual(rci.get_val(point4), 0.5, delta=0.03)

    def test_simple_distance_setres(self):
        """ Very simple test with a raster object like thus:
             1  2  3  4
             5  6  7  8
             9  10 11 12
             13 14 15 16
            (on a 10x10 grid, not 4x4 as above...only so much space
            in comments!)

            LLC is 0,0 and upper right is 4,4. (hence dx is 0.25)
            The data are stored in cell centres and we ask for a few coords.
            We check that the value is 1 at the distance, 0 at the boundary.
            Finally, test that the file is written and readable
            """
        rbuff = CreateBuffer(test_file_name1, 1.5, over=30)
        point1 = [0.03, 3.85]  # should return ~0
        point2 = [1.65, 2]  # should be 1
        point3 = [2, 2]  # should be 1
        point4 = [0.76, 2]  # should be ~0.5
        rbuff.make_buffer(temp_file)
        # we now read in the buffer using the rasterinterpolator class
        rci = RasterInterpolator(temp_file)
        rci.set_band()
        self.assertAlmostEqual(rci.get_val(point1), 0.0, delta=0.01)
        self.assertEqual(rci.get_val(point2), 1.0)
        self.assertEqual(rci.get_val(point3), 1.0)
        self.assertAlmostEqual(rci.get_val(point4), 0.5, delta=0.01)

    def test_simple_distance_nan(self):
        """ Very simple test with a raster object like thus:
             1  2  3  4
             5  6  7  8
             9  NAN NAN 12
             13 14 15 16
            (on a 20x20 grid, not 4x4 as above...only so much space
            in comments!). The slightly-left-of-centre has been
            replaced by NaN

            LLC is 0,0 and upper right is 4,4. (hence dx is 0.25)
            The data are stored in cell centres and we ask for a few coords.
            We check that the value is 1 at the distance, 0 at the boundary
            and, for this test, the value is 0 in the centre.

            The NaNs are in the centre, so the buffer is expanded by one pixel
            automatically to prevent interpolation issues
            """
        rbuff = CreateBuffer(test_file_name2, 0.5)
        point1 = [0.2, 3.85]  # should return ~0
        point2 = [0.8, 2]  # should be 0.2
        point3 = [1.7, 2]  # should be 0 (in the centre)
        rbuff.make_buffer(temp_file)
        # we now read in the buffer using the rasterinterpolator class
        rci = RasterInterpolator(temp_file)
        rci.set_band()
        self.assertAlmostEqual(rci.get_val(point1), 0.0, delta=0.03)
        self.assertAlmostEqual(rci.get_val(point2), 0.2, delta=0.03)
        self.assertEqual(rci.get_val(point3), 0.0)


if __name__ == '__main__':
    unittest.main()
